/******************************************************************************
 * Copyright 2024 ztsh.eu                                                     *
 *                                                                            *
 * Licensed under the Apache License, Version 2.0 (the "License");            *
 * you may not use this file except in compliance with the License.           *
 * You may obtain a copy of the License at                                    *
 *                                                                            *
 *     http://www.apache.org/licenses/LICENSE-2.0                             *
 *                                                                            *
 * Unless required by applicable law or agreed to in writing, software        *
 * distributed under the License is distributed on an "AS IS" BASIS,          *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   *
 * See the License for the specific language governing permissions and        *
 * limitations under the License.                                             *
 ******************************************************************************/

package eu.ztsh.lib.time;

import java.time.LocalDate;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Objects;
import java.util.stream.Stream;

import static eu.ztsh.lib.time.Formatters.dayMonth;

/**
 * A range of dates.
 *
 * @param from Start date (inclusive)
 * @param to   End date (inclusive)
 */
public record TimeRange(LocalDate from, LocalDate to) {

    public TimeRange {
        to = Objects.requireNonNullElse(to, LocalDate.now());
    }

    public TimeRange(LocalDate from) {
        this(from, null);
    }

    public Iterator<LocalDate> iterator() {
        return new TimeRangeIterator(from, to);
    }

    public List<LocalDate> getDates() {
        return from.datesUntil(to.plusDays(1)).toList();
    }

    public long getLength() {
        return getDates().size();
    }

    public TimeRange expand(LocalDate target) {
        if (target.isBefore(from)) {
            return new TimeRange(target, to);
        } else if (target.isAfter(to)) {
            return new TimeRange(from, target);
        }
        return this;
    }

    public TimeRange merge(TimeRange other) {
        var merged = Stream.concat(getDates().stream(), other.getDates().stream()).sorted().distinct().toList();
        var first = merged.get(0);
        var last = merged.get(merged.size() - 1);
        if (first.until(last).getDays() > merged.size()) {
            throw new IllegalArgumentException("Adjacent ranges");
        }
        return new TimeRange(first, last);
    }

    @Override
    public boolean equals(Object o) {
        return this == o || (o instanceof TimeRange that)
                && this.from.equals(that.from)
                && this.to.equals(that.to);
    }

    @Override
    public int hashCode() {
        return Objects.hash(from, to);
    }

    @Override
    public String toString() {
        return dayMonth.format(from) + "-" + dayMonth.format(to);
    }

    private static class TimeRangeIterator implements Iterator<LocalDate> {

        private final LocalDate to;
        private LocalDate now;

        private TimeRangeIterator(LocalDate from, LocalDate to) {
            this.to = to;
            now = from;
        }

        @Override
        public boolean hasNext() {
            return !now.isAfter(to);
        }

        @Override
        public LocalDate next() {
            if (!hasNext()) {
                throw new NoSuchElementException("Out of range");
            }
            now = now.plusDays(1);
            return now.minusDays(1);
        }

    }

}
