/******************************************************************************
 * Copyright 2024 ztsh.eu                                                     *
 *                                                                            *
 * Licensed under the Apache License, Version 2.0 (the "License");            *
 * you may not use this file except in compliance with the License.           *
 * You may obtain a copy of the License at                                    *
 *                                                                            *
 *     http://www.apache.org/licenses/LICENSE-2.0                             *
 *                                                                            *
 * Unless required by applicable law or agreed to in writing, software        *
 * distributed under the License is distributed on an "AS IS" BASIS,          *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   *
 * See the License for the specific language governing permissions and        *
 * limitations under the License.                                             *
 ******************************************************************************/

package eu.ztsh.lib.time.util;

import eu.ztsh.lib.time.TimeRange;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.List;

/**
 * Collection of common DateTime oriented functions
 */
public class TimeUtil {

    public static LocalDate today() {
        return LocalDate.now();
    }

    public static LocalDate yesterday() {
        return today().minusDays(1);
    }

    public static LocalDate convert(Date dateToConvert) {
        return dateToConvert.toInstant()
                .atZone(ZoneId.systemDefault())
                .toLocalDate();
    }

    public static Date convert(LocalDate dateToConvert) {
        return Date.from(dateToConvert.atStartOfDay()
                .atZone(ZoneId.systemDefault())
                .toInstant());
    }

    public static LocalDateTime convertFull(Date dateToConvert) {
        return dateToConvert.toInstant()
                .atZone(ZoneId.systemDefault())
                .toLocalDateTime();
    }

    public static boolean isMonday(LocalDate today) {
        return today.getDayOfWeek() == DayOfWeek.MONDAY;
    }

    public static boolean isWeekend(LocalDate today) {
        return today.getDayOfWeek() == DayOfWeek.SATURDAY
                || today.getDayOfWeek() == DayOfWeek.SUNDAY;
    }

    public static boolean isMonthBeginning(LocalDate today) {
        return !today.minusWeeks(1).getMonth().equals(today.getMonth());
    }

    public static LocalDate getLastMonday(LocalDate today) {
        return today.with(TemporalAdjusters.previousOrSame(DayOfWeek.MONDAY));
    }

    public static LocalDate getNextSunday(LocalDate today) {
        return today.with(TemporalAdjusters.nextOrSame(DayOfWeek.SUNDAY));
    }

    public static LocalDate getNextFriday(LocalDate today) {
        return today.with(TemporalAdjusters.nextOrSame(DayOfWeek.FRIDAY));
    }

    public static LocalDate getLastFriday(LocalDate today) {
        return today.with(TemporalAdjusters.previous(DayOfWeek.FRIDAY));
    }

    public static LocalDate getFirstDayOfMonth(LocalDate today) {
        return today.with(TemporalAdjusters.firstDayOfMonth());
    }

    public static LocalDate getLastDayOfMonth(LocalDate today) {
        return today.with(TemporalAdjusters.lastDayOfMonth());
    }

    public static long countUntil(List<TimeRange> ranges, LocalDate today) {
        return ranges.stream()
                .map(TimeRange::getDates)
                .flatMap(List::stream)
                .filter(date -> date.getMonth().equals(today.getMonth()) && (date.isBefore(today) || date.equals(today)))
                .count();
    }

    private TimeUtil() {
        throw new IllegalStateException("Utility class");
    }

}
