/******************************************************************************
 * Copyright 2024 ztsh.eu                                                     *
 *                                                                            *
 * Licensed under the Apache License, Version 2.0 (the "License");            *
 * you may not use this file except in compliance with the License.           *
 * You may obtain a copy of the License at                                    *
 *                                                                            *
 *     http://www.apache.org/licenses/LICENSE-2.0                             *
 *                                                                            *
 * Unless required by applicable law or agreed to in writing, software        *
 * distributed under the License is distributed on an "AS IS" BASIS,          *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   *
 * See the License for the specific language governing permissions and        *
 * limitations under the License.                                             *
 ******************************************************************************/

package eu.ztsh.lib.time;

import org.junit.jupiter.api.Test;

import java.time.LocalDate;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

class TimeRangeTest {

    private final LocalDate date1 = LocalDate.of(2023, 10, 10);
    private final LocalDate date2 = LocalDate.of(2023, 10, 12);
    private final LocalDate date3 = LocalDate.of(2023, 10, 13);
    private final LocalDate date4 = LocalDate.of(2023, 10, 15);

    @Test
    void lengthTest() {
        var range = new TimeRange(date1, date2);
        assertThat(range.getDates()).hasSize(3);
        assertThat(range.getLength()).isEqualTo(3);
    }

    @Test
    void equalityTest() {
        var range1 = new TimeRange(date1, date2);
        var range2 = new TimeRange(date1, date2);
        assertThat(range1).isEqualTo(range2);
    }

    @Test
    void nonEqualityTest() {
        var range1 = new TimeRange(date1, date2);
        var range2 = new TimeRange(date1, date3);
        assertThat(range1).isNotEqualTo(range2);
    }

    @Test
    void expandAfterTest() {
        var test = new TimeRange(date1, date2);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date3);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void expandBeforeTest() {
        var test = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void expandInternalTest() {
        var test = new TimeRange(date1, date3);
        var expected = new TimeRange(date1, date3);
        var result = test.expand(date2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeAfterTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date3, date4);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeBeforeTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date3, date4);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeIntersectingBeforeTest() {
        var test1 = new TimeRange(date1, date3);
        var test2 = new TimeRange(date2, date4);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeIntersectionAfterTest() {
        var test1 = new TimeRange(date1, date3);
        var test2 = new TimeRange(date2, date4);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeMinorContainingTest() {
        var test1 = new TimeRange(date1, date4);
        var test2 = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date4);
        var result = test1.merge(test2);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeMajorContainingTest() {
        var test1 = new TimeRange(date1, date4);
        var test2 = new TimeRange(date2, date3);
        var expected = new TimeRange(date1, date4);
        var result = test2.merge(test1);
        assertThat(result).isEqualTo(expected);
    }

    @Test
    void mergeAdjacentTest() {
        var test1 = new TimeRange(date1, date2);
        var test2 = new TimeRange(date4, date4);
        assertThatThrownBy(() -> test1.merge(test2)).isInstanceOf(IllegalArgumentException.class);
    }

}